import { Metadata, Singleton } from '@zeedhi/core';
import { Modal, Select, TextInput, Tree } from '@zeedhi/common';
import Store from '@/store';
import { PmLibDialogService, PmLibFormSave } from '@zeedhi/pm-lib';
import { NodeType } from '@/services/AclLabels/NodeType';
import { AclLabelsService } from '@/services/AclLabels/AclLabelsService';
import Messages from '@/services/Messages';

@Singleton
export class AclLabelsEditLabelModalController {
	get editLabelModal(): Modal {
		return Store.getters['AclLabels/getEditLabelModal'];
	}

	get aclLabelsTreeComponent(): Tree {
		return Store.getters['AclLabels/getAclLabelsTreeComponent'];
	}

	get nodeTypeValue() {
		return Store.getters['AclLabels/getNodeTypeValue'];
	}

	get hasChanges(): boolean {
		return Store.getters['AclLabels/getHasChanges'];
	}

	set hasChanges(value: boolean) {
		Store.commit('AclLabels/setHasChanges', value);
	}

	get currentNodeEdit(): any {
		return Store.getters['AclLabels/getCurrentNodeEdit'];
	}

	public editNode?: any;

	public editNodeBackup: any = {};

	public onMountedEditLabelModalForm() {
		const editLabelForm = Metadata.getInstance<PmLibFormSave>('editLabelModalForm');
		const typeField = editLabelForm.getChildInstance<Select>('type');
		const labelField = editLabelForm.getChildInstance<TextInput>('label');
		const isGrouped: boolean = this.nodeTypeValue === NodeType.grouped;
		labelField.readonly = isGrouped;
		if (isGrouped) {
			labelField.removeValidation('required');
		} else {
			labelField.addValidation('required');
			labelField.value = this.currentNodeEdit.data?.label;
		}
		typeField.setValue(this.nodeTypeValue);
	}

	public closeEditLabelModalForm() {
		this.editLabelModal?.hide();
	}

	public getEditLabelModalForm(): PmLibFormSave {
		return Metadata.getInstance<PmLibFormSave>('editLabelModalForm');
	}

	public saveEditLabelModalForm() {
		const formEdit = this.getEditLabelModalForm();
		if (formEdit.validate() && this.validateDuplicateLabels(formEdit)) {
			const newItem = this.getNewItem(this.getEditLabelModalForm());
			this.aclLabelsTreeComponent.updateNodeData(this.editNode, newItem);
			this.hasChanges = true;
			this.editLabelModal?.hide();
			this.editNodeBackup = {};
		}
	}

	private validateDuplicateLabels(editLabelForm: PmLibFormSave): boolean {
		if (editLabelForm.value.type === NodeType.rule && this.editNodeBackup.data.label !== editLabelForm.value.label) {
			const { label } = editLabelForm.value;
			const allLabels = AclLabelsService.getAllLabelsInNodes(this.aclLabelsTreeComponent.nodes);
			if (allLabels.includes(label)) {
				PmLibDialogService.showError(Messages.MSG_DUPLICATE_LABEL_SAVE);
			}
			return !allLabels.includes(label);
		}

		return true;
	}

	public getNewItem(form: PmLibFormSave): any {
		const newItem: any = {
			title: form.value.name,
			data: {
				type: form.value.type,
			},
		};

		if (this.editNode.data.version) {
			newItem.data.version = this.editNode.data.version;
		}

		if (form.value.label && form.value.type !== NodeType.grouped) {
			newItem.data.label = form.value.label;
		}

		return newItem;
	}
}
