import { InstanceService } from '@/services/InstanceService';
import { AlertServiceCustom } from '@/services/AlertServiceCustom';
import { Utilities } from '@/services/Utilities';
import { DialogServiceCustom } from '@/services/DialogServiceCustom';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Http } from '@zeedhi/core';
import { DialogService, LoadingService } from '@zeedhi/common';
import Messages from '@/services/Messages';

export class DMBudgetSoliController {
	public callRelationship = '';

	public callType = 'I';

	private validateEditBudget = true;

	public async beforeCancelGridBudgetItensSoli() {
		const budgetItensGridInstance = InstanceService.getInstanceTekGrid('gridbudgetItensSoli');
		await budgetItensGridInstance.datasource.get();
	}

	public async beforeCancelGridBudgetInstallmentsSoli() {
		const budgetItensGridInstance = InstanceService.getInstanceTekGrid('gridBudgetInstallmentsSoli');
		await budgetItensGridInstance.datasource.get();
	}

	public validateNumberParcel(parcel: number) {
		let validate = true;
		if (parcel >= 100) {
			AlertServiceCustom.customErrorAlert(Messages.ERR_MANY_PLOTS);
			validate = false;
		} else if (parcel <= 0) {
			AlertServiceCustom.customErrorAlert(Messages.ERR_ZERO_NUM_PLOTS);
			validate = false;
		} else if (!parcel) {
			AlertServiceCustom.customErrorAlert(Messages.ERR_INVALID_NUM_PLOTS);
			validate = false;
		}
		return validate;
	}

	public setDayPaymentDown(row: any, widgetPlots: any, prevRowDaysPayment: any, daysPaymentAdd: any) {
		let forPrevRowDaysPayment = prevRowDaysPayment;
		for (let x = parseInt(row.ORDER_PLOTS_NUM) - 2; x >= 0; x--) {
			const nextRow = widgetPlots.datasource.data[x];
			if (nextRow && nextRow.DAYS_PAYMENT >= forPrevRowDaysPayment) {
				nextRow.DAYS_PAYMENT = parseInt(nextRow.DAYS_PAYMENT) + parseInt(daysPaymentAdd);
				nextRow.DAYS_PAYMENT_COPY = nextRow.DAYS_PAYMENT;
				nextRow.DAYS_PAYMENT_inline = nextRow.DAYS_PAYMENT;
				nextRow.changed = true;
				forPrevRowDaysPayment = nextRow.DAYS_PAYMENT;
			}
		}
	}

	public setDayPaymentUp(row: any, widgetPlots: any, prevRowDaysPayment: any, daysPaymentAdd: any) {
		let forPrevRowDaysPayment = prevRowDaysPayment;
		for (let x = parseInt(row.ORDER_PLOTS_NUM); x <= widgetPlots.datasource.data.length; x++) {
			const nextRow = widgetPlots.datasource.data[x];
			if (nextRow && nextRow.DAYS_PAYMENT <= forPrevRowDaysPayment) {
				nextRow.DAYS_PAYMENT = parseInt(nextRow.DAYS_PAYMENT) + parseInt(daysPaymentAdd);
				nextRow.DAYS_PAYMENT_COPY = nextRow.DAYS_PAYMENT;
				nextRow.DAYS_PAYMENT_inline = nextRow.DAYS_PAYMENT;
				nextRow.changed = true;
				forPrevRowDaysPayment = nextRow.DAYS_PAYMENT;
			}
		}
	}

	public changeDayPaymentGridBudgetInstallmentsSoli(params: any) {
		const { row } = params;
		row.DAYS_PAYMENT_inline = params.component.value;
		row.DAYS_PAYMENT_COPY = row.DAYS_PAYMENT;
		row.DAYS_PAYMENT = params.component.value;
		const widgetPlots = InstanceService.getInstanceTekGrid('gridBudgetInstallmentsSoli');
		widgetPlots.datasource.data[row.ORDER_PLOTS_NUM - 1].DAYS_PAYMENT = row.DAYS_PAYMENT;
		const daysPaymentAdd = row.DAYS_PAYMENT_inline - row.DAYS_PAYMENT_COPY;
		row.DAYS_PAYMENT_COPY = row.DAYS_PAYMENT_inline;
		const prevRowDaysPayment = row.DAYS_PAYMENT_inline;
		if (daysPaymentAdd > 0) {
			this.setDayPaymentUp(row, widgetPlots, prevRowDaysPayment, daysPaymentAdd);
		} else {
			this.setDayPaymentDown(row, widgetPlots, prevRowDaysPayment, daysPaymentAdd);
		}
		this.updateGroupDataBudgetPLotsDayPayment(widgetPlots);
	}

	public setValuesAddParcel(parcel: any, widgetPlots: TekGrid) {
		const diff = parseFloat(parcel) - (widgetPlots.datasource.data.length) + 1;
		const valueTotal = Math.trunc(widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1].VALUE_PLOTS * 100) / 100;
		const value = Math.trunc((valueTotal / diff) * 100) / 100;
		const rest = valueTotal - ((value * 100 * diff) / 100);
		widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1].VALUE_PLOTS = value;
		widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1].changed = true;
		return [value, rest];
	}

	public removeParcel(widgetPlots: TekGrid) {
		const row = widgetPlots.datasource.data.pop();
		widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1]
			.VALUE_PLOTS = ((parseFloat(widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1]
			.VALUE_PLOTS) * 100) + parseFloat(row?.VALUE_PLOTS) * 100) / 100;
		widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1].changed = true;
	}

	public addParcel(widgetPlots: TekGrid, value: any, parcelLabel: string, parcel: number | null, rest: any) {
		const numParcel = widgetPlots.datasource.data.length + 1;
		const newRow = {
			VALUE_PLOTS: parseFloat(value),
			changed: true,
			ORDER_PLOTS: `${numParcel}° ${parcelLabel}`,
			ORDER_PLOTS_NUM: numParcel,
			DAYS_PAYMENT: parseInt(widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1].DAYS_PAYMENT, 10) + 30,
			BUDGET_ID: widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1].BUDGET_ID,
		};
		if (parcel === widgetPlots.datasource.data.length) {
			newRow.VALUE_PLOTS += parseFloat(rest);
		}
		widgetPlots.datasource.data.push(newRow);
	}

	public async generateProposalInSMClickGridbudgetItensSoli() {
		let response;
		try {
			LoadingService.show();
			response = await Http.post('/soliGenerateProposalValidate', {
				CALL_RELATIONSHIP: this.callRelationship,
			});
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
		if (response && response.data && response.data.type) {
			const typeMessage = response.data.type;
			if (typeMessage === 'ERROR') {
				AlertServiceCustom.customErrorAlert(Messages.FASE_ERROR_PROPOSAL);
			} else if (typeMessage === 'ALERT') {
				const message = Messages.ALERT_PROPOSTA_CREATE_SMT;
				const btnYes = [{
					name: Utilities.getNameDynamic('yes'),
					component: 'ZdButton',
					label: 'Sim',
					outline: false,
					events: {
						click: (() => {
							DialogService.hide();
							this.createProposalSM();
						}),
					},
				}];
				DialogServiceCustom.confirmMessage(message, btnYes);
			} else {
				await this.createProposalSM();
			}
		}
		LoadingService.hide();
	}

	public async createProposalSM() {
		try {
			LoadingService.show();
			await Http.post('/soliGenerateProposal', {
				CALL_RELATIONSHIP: this.callRelationship,
			});
		} catch (e) {
			LoadingService.hide();
			DialogServiceCustom.showErrorBackend(e);
		}
	}

	public async quantityPlotsChangeFormBudgetInstallmentsSoli() {
		const widgetPlots = InstanceService.getInstanceTekGrid('gridBudgetInstallmentsSoli');
		const formBudgetInstallmentsSoli = InstanceService.getInstanceZdForm('formBudgetInstallmentsSoli');
		const parcel = parseInt(formBudgetInstallmentsSoli.value.QUANTITY_PLOTS, 10);
		let value = null;
		let rest = null;
		const parcelLabel = 'Parcelas';
		if (this.validateNumberParcel(parcel)) {
			if (parcel === 1) {
				widgetPlots.datasource.data = [widgetPlots.datasource.data[0]];
				widgetPlots.datasource.data[0].VALUE_PLOTS = widgetPlots.datasource.data[0].PROPOSAL_VALUE;
			} else {
				if (parcel > widgetPlots.datasource.data.length) {
					const ret = this.setValuesAddParcel(parcel, widgetPlots);
					value = ret[0];
					rest = ret[1];
				}
				while (parcel !== widgetPlots.datasource.data.length) {
					if (parcel < widgetPlots.datasource.data.length) {
						this.removeParcel(widgetPlots);
					} else {
						this.addParcel(widgetPlots, value, parcelLabel, parcel, rest);
					}
				}
			}
			this.widgetPlotsUpdateGroups();
			widgetPlots.editing = true;
		}
	}

	public widgetPlotsUpdateGroups() {
		const widget = InstanceService.getInstanceTekGrid('gridBudgetInstallmentsSoli');
		const groupedData = widget.groupedData.filter((row) => {
			let filter = false;
			if (row.group) {
				filter = row.group;
			}
			return filter;
		});
		let total = 0;
		widget.datasource.data.forEach((row) => {
			groupedData.push(row);
			total += row.VALUE_PLOTS;
		});
		widget.groupedData.forEach((row) => {
			if (row.groupFooter) {
				row.VALUE_PLOTS = total;
				groupedData.push(row);
			}
		});
		widget.groupedData = groupedData;
	}

	public async beforeSaveGridBudgetItensSoli(params: any) {
		params.event?.preventDefault();
		LoadingService.show();
		try {
			const budgetItensGridInstance = InstanceService.getInstanceTekGrid('gridbudgetItensSoli');
			const formMaintenanceCostSoli = InstanceService.getInstanceZdForm('formMaintenanceCostSoli');
			await Http.post('/SoliBudgetSave', {
				CALL_RELATIONSHIP: this.callRelationship,
				CALL_TYPE: this.callType,
				DATA: budgetItensGridInstance.groupedData,
				DATA_MAINTENANCE: formMaintenanceCostSoli.value,
			});
			await budgetItensGridInstance.reload();
			await budgetItensGridInstance.cancelEditedRows();
			LoadingService.hide();
		} catch (e) {
			LoadingService.hide();
		}
	}

	public async beforeSaveGridBudgetInstallmentsSoli(params: any) {
		params.event?.preventDefault();
		const budgetItensGridInstance = InstanceService.getInstanceTekGrid('gridBudgetInstallmentsSoli');
		if (this.validateSaveWidgetPlots(budgetItensGridInstance)) {
			LoadingService.show();
			try {
				await Http.post('/SoliBudgetInstallmentsSave', {
					CALL_RELATIONSHIP: this.callRelationship,
					DATA: budgetItensGridInstance.datasource.data,
				});
				await budgetItensGridInstance.reload();
				await budgetItensGridInstance.cancelEditedRows();
				LoadingService.hide();
			} catch (e) {
				LoadingService.hide();
			}
		}
	}

	public changeValuePlotsGridBudgetInstallmentsSoli(params: any) {
		const { row } = params;
		row.VALUE_PLOTS = params.component.value;
		const budgetItensGridInstance = InstanceService.getInstanceTekGrid('gridBudgetInstallmentsSoli');
		budgetItensGridInstance.datasource.data[row.ORDER_PLOTS_NUM - 1].VALUE_PLOTS = row.VALUE_PLOTS;
		this.updateGroupDataBudgetPLots(budgetItensGridInstance);
	}

	public async changeTabBudgetSoli(params: any) {
		try {
			this.callRelationship = params.component.events.callRelationship();
			const tabBudgetSoli = InstanceService.getInstanceTabs('tabBudgetSoli');
			const tab = tabBudgetSoli.tabs[tabBudgetSoli.activeTab];
			setTimeout(async () => {
				if (tab.events && tab.events.change && typeof tab.events.change === 'function') {
					await tab.events.change(params);
				}
			}, 100);
		} catch (e) {
			setTimeout(async () => {
				await this.changeTabBudgetSoli(params);
			}, 1000);
		}
	}

	public validadetotalparcel(widgetPlots: TekGrid) {
		let total = 0;
		const proposalValue = widgetPlots.datasource.data[0].PROPOSAL_VALUE;
		widgetPlots.datasource.data.forEach((row: any) => {
			if (typeof row.VALUE_PLOTS === 'string') {
				row.VALUE_PLOTS = row.VALUE_PLOTS.replace(/[.]/g, '').replace(',', '.');
			}
			total += row.VALUE_PLOTS * 100;
		});
		return total === proposalValue * 100;
	}

	public validateSaveWidgetPlots(widgetPlots: TekGrid) {
		let validate = true;
		if (!widgetPlots.datasource.data.length) {
			AlertServiceCustom.customErrorAlert(Messages.ERR_EMPTY_PLOTS);
			validate = false;
		} else if (!this.validadetotalparcel(widgetPlots)) {
			const mess = `${Messages.ERR_TOTAL_PLOTS} (${widgetPlots.datasource.data[0].PROPOSAL_VALUE_VIEW.trim()}).`;

			AlertServiceCustom.customErrorAlert(mess);
			validate = false;
		}
		return validate;
	}

	public async changeTabFrameBudgetInstallmentsSoli() {
		const gridBudgetInstallmentsSoli = InstanceService.getInstanceTekGrid('gridBudgetInstallmentsSoli');
		const formBudgetInstallmentsSoli = InstanceService.getInstanceZdForm('formBudgetInstallmentsSoli');
		await gridBudgetInstallmentsSoli.datasource.get();
		formBudgetInstallmentsSoli.value.QUANTITY_PLOTS = gridBudgetInstallmentsSoli.datasource.data.length;
		this.changeTabFrameBudgetItensSoli();
	}

	public async changeTabFrameBudgetItensSoli() {
		LoadingService.show();
		try {
			const budgetItensGridInstance = InstanceService.getInstanceTekGrid('gridbudgetItensSoli');
			const formMaintenanceCostSoli = InstanceService.getInstanceZdForm('formMaintenanceCostSoli');
			formMaintenanceCostSoli.children.forEach((row) => {
				formMaintenanceCostSoli.value[row.name] = 0;
			});
			budgetItensGridInstance.datasource.get();
			const response = await Http.post('/maintenanceCostSoli', {
				CALL_RELATIONSHIP: this.callRelationship,
				CALL_TYPE: this.callType,
			});
			formMaintenanceCostSoli.children.forEach((row) => {
				if (response && response.data && response.data.data && response.data.data[row.name]) {
					formMaintenanceCostSoli.value[row.name] = response.data.data[row.name];
				} else {
					formMaintenanceCostSoli.value[row.name] = 0;
				}
			});
			formMaintenanceCostSoli.value.changed = false;
			formMaintenanceCostSoli.value.Soli_ID = this.callRelationship;
			this.validateEditBudget = !!formMaintenanceCostSoli.value.VALIDATE_EDIT;
			if (!formMaintenanceCostSoli.value.VALIDATE_EDIT) {
				AlertServiceCustom.customErrorAlert(Messages.ALE_TASK_EDITING_BUDGE_WIDGET);
			}
			LoadingService.hide();
		} catch (e) {
			LoadingService.hide();
		}
	}

	public formMaintenanceCostSoliChangeValueDiscount() {
		const formMaintenanceCostSoli = InstanceService.getInstanceZdForm('formMaintenanceCostSoli');
		let discountValue = formMaintenanceCostSoli.value.VALUE_DISCOUNT;
		const total = formMaintenanceCostSoli.value.MANUT_VALUE;
		if (parseFloat(discountValue) > parseFloat(total)) {
			discountValue = total;
		}
		const percent = parseFloat(((discountValue / total) * 100).toFixed(2));
		formMaintenanceCostSoli.value.TOTAL_VALUE = total - discountValue;
		formMaintenanceCostSoli.value.PERCENTAGE_DISCOUNT = percent;
		const widget = InstanceService.getInstanceTekGrid('gridbudgetItensSoli');
		widget.editing = true;
		formMaintenanceCostSoli.value.changed = true;
	}

	public formMaintenanceCostSoliChangePercentDiscount() {
		const formMaintenanceCostSoli = InstanceService.getInstanceZdForm('formMaintenanceCostSoli');
		let percentValue = formMaintenanceCostSoli.value.PERCENTAGE_DISCOUNT;
		const total = formMaintenanceCostSoli.value.MANUT_VALUE;
		if (percentValue > 100) {
			percentValue = 100;
			formMaintenanceCostSoli.value.PERCENTAGE_DISCOUNT = 100;
		}
		const discValue = (percentValue * total) / 100;
		formMaintenanceCostSoli.value.TOTAL_VALUE = total - discValue;
		formMaintenanceCostSoli.value.VALUE_DISCOUNT = discValue;
		const widget = InstanceService.getInstanceTekGrid('gridbudgetItensSoli');
		widget.editing = true;
		formMaintenanceCostSoli.value.changed = true;
	}

	public async cancelEditedRows() {
		const budgetItensGridInstance = InstanceService.getInstanceTekGrid('gridbudgetItensSoli');
		await budgetItensGridInstance.reload();
		await budgetItensGridInstance.cancelEditedRows();
	}

	public changeProposalQuantityGridBudgetItensSoli(params: any) {
		const codeMesager = <string>params.row.CODE_MESAGER;
		let message = '';
		if (codeMesager && codeMesager.startsWith('ERRO')) {
			message = AlertServiceCustom.getMessageByCode(codeMesager);
			AlertServiceCustom.customErrorAlert(message);
			this.cancelEditedRows();
		} else {
			if (codeMesager) {
				message = AlertServiceCustom.getMessageByCode(codeMesager);
				AlertServiceCustom.customAlert(message);
			}
			let proposalQuantity = params.component.value;
			if (!proposalQuantity) {
				proposalQuantity = 0;
			} else if (proposalQuantity < 0) {
				proposalQuantity *= -1;
			}
			const currentRow = params.row;
			currentRow.PROPOSAL_QUANTITY = proposalQuantity;
			const widget = InstanceService.getInstanceTekGrid('gridbudgetItensSoli');

			if (currentRow.BUDGET_ITEM_ID === '999999XXXX') {
				if (proposalQuantity > 100) {
					currentRow.PROPOSAL_QUANTITY = 100;
				}
			} else {
				currentRow.PROPOSAL_VALUE = proposalQuantity * currentRow.ITEM_VALUE_TRUE;
				this.updateGroupDataBudget(widget);
			}

			const indexSubTotalDescon = widget.groupedData.length - 2;
			const indexDescon = widget.groupedData.length - 3;
			const indexSubTotal = widget.groupedData.length - 5;
			const discountPercent = parseFloat(widget.groupedData[indexDescon].PROPOSAL_QUANTITY);
			const subTotalProposal = parseFloat(widget.groupedData[indexSubTotal].PROPOSAL_VALUE);
			const discountValue = Number(((subTotalProposal * discountPercent) / 100).toFixed(2)) * -1;
			widget.groupedData[indexSubTotalDescon].PROPOSAL_VALUE = discountValue;
			widget.groupedData[indexDescon].PROPOSAL_VALUE = Number(discountValue);

			this.updateGroupDataBudget(widget);
		}
	}

	public updateGroupDataBudgetPLots(widget: TekGrid) {
		const indexTotal = widget.groupedData.length - 1;
		const indexSubTotal = widget.groupedData.length - 2;
		let total = 0;
		widget.groupedData.forEach((row, index) => {
			if (!row.group) {
				if (index !== indexSubTotal
					&& index !== indexTotal
				) {
					total += row.VALUE_PLOTS;
				} else {
					row.VALUE_PLOTS = total;
				}
			}
		});
	}

	public updateGroupDataBudgetPLotsDayPayment(widget: TekGrid) {
		widget.groupedData.forEach((row) => {
			if (!row.group && row.DAYS_PAYMENT && row.ORDER_PLOTS_NUM) {
				row.DAYS_PAYMENT = widget.datasource.data[row.ORDER_PLOTS_NUM - 1].DAYS_PAYMENT;
			}
		});
	}

	public updateGroupDataBudget(widget: TekGrid) {
		const indexTotal = widget.groupedData.length - 1;
		const indexSubTotalDescon = widget.groupedData.length - 2;
		const indexDescon = widget.groupedData.length - 3;
		const indexSubTotal = widget.groupedData.length - 5;
		let total = 0;
		let totalEstimat = 0;
		widget.groupedData[0].changed = true;
		widget.groupedData.forEach((row, index) => {
			if (!row.group) {
				if (index !== indexSubTotal
					&& index !== indexSubTotalDescon
					&& index !== indexTotal
				) {
					total += row.PROPOSAL_VALUE;
					if (!row.NOT_NEW && index !== indexDescon) {
						row.PROPOSAL_VALUE_ESTIMAT = row.PROPOSAL_VALUE;
						row.PROPOSAL_QUANTITY_ESTIMAT = row.PROPOSAL_QUANTITY;
					}
					totalEstimat += row.PROPOSAL_VALUE_ESTIMAT;
				} else if (index === indexSubTotalDescon) {
					row.PROPOSAL_VALUE = widget.groupedData[indexDescon].PROPOSAL_VALUE;
				} else {
					row.PROPOSAL_VALUE = total;
				}
				if (index === indexSubTotal || index === indexTotal) {
					row.PROPOSAL_VALUE_ESTIMAT = totalEstimat;
				}
			}
		});
		const formMaintenanceCostSoli = InstanceService.getInstanceZdForm('formMaintenanceCostSoli');
		formMaintenanceCostSoli.value.MANUT_VALUE = Number(((total * formMaintenanceCostSoli.value.MAINTENANCE_PERCENTAGE) / 100).toFixed(2));
		this.formMaintenanceCostSoliChangePercentDiscount();
	}
}
